﻿using UnityEngine;
using System.Collections;

public class AreaScript : MonoBehaviour {

    public GameObject[] tileColors;
    public static Tile[,] tiles = null;

    public static float tileSizeX = 1.28f;
    public static float tileSizeY = 1.28f;

	// Use this for initialization
	void Start () {
        GenerateLevel();
	}
	
	// Update is called once per frame
	void Update () {
	
	}

    public void GenerateLevel()
    {
        int levelSizeX = 16;
        int levelSizeY = 16;

        tiles = new Tile[levelSizeX, levelSizeY];

        for (int iy = 0; iy < levelSizeY; iy++)
        {
            for (int ix = 0; ix < levelSizeX; ix++)
            {
                GameObject newTile = ChangeTile(ix, iy, "random");
                tiles[ix, iy].color = newTile.name.Substring(5).Replace("(Clone)", ""); 
            }
        }

        //Then bias the level a bit
        int randomBiasing = 8;
        int randomBiasRepetition = 200;
        for (int i = 0; i < randomBiasing; i++)
        {
            int baseX = Random.Range((int)0, levelSizeX);
            int baseY = Random.Range((int)0, levelSizeY);

            int amountOfRepetitions = Random.Range((int)1, (int)randomBiasRepetition);
            for (int j = 0; j < amountOfRepetitions; j++)
            {
                string targetColor = tiles[baseX, baseY].color;

                int xMod = Random.Range((int)-1, (int)2); //Exclusive max
                int yMod = Random.Range((int)-1, (int)2);

                int targetX = baseX + xMod;
                int targetY = baseY + yMod;

                if (targetX >= 0 && targetX < levelSizeX)
                {
                    if (targetY >= 0 && targetY < levelSizeY)
                    {
                        ChangeTile(targetX, targetY, targetColor);
                        baseX = targetX;
                        baseY = targetY;
                    }
                }
            }
        }
    }

    public GameObject ChangeTile(int x, int y, string color)
    {
        if (tiles[x, y] != null)
        {
            Destroy(tiles[x, y].gameObject);
        }
        Vector3 targetPosition = new Vector2(x * tileSizeX, y * tileSizeY);
        GameObject newTile = (GameObject)GameObject.Instantiate(GetColoredTile(color));
        newTile.transform.localPosition = targetPosition;
        tiles[x, y] = (Tile)newTile.GetComponent("Tile");
        tiles[x, y].color = color;

        return newTile;
    }

    public GameObject GetColoredTile(string color)
    {
        if (color == "random")
        {
            return tileColors[Random.Range((int)0, tileColors.Length)];
        }
        else
        {
            for (int i = 0; i < tileColors.Length; i++)
            {
                if (tileColors[i].name.Substring(5).Replace("(Clone)", "") == color) return tileColors[i]; //Assuming tile is named "tile_"
            }
        }

        return null;
    }
}
