using UnityEngine;
using System.Collections;
using XInputDotNetPure;
using System.Collections.Generic;

public class PlayerController : MonoBehaviour {

	/*
	D-Pad:
	Horizontal = 6th axis
	Vertical = 7th axis
	
	Left Analog:
	Horizontal = X axis
	Vertical = Y axis
	
	Right Analog:
	Horizontal = 4th axis
	Vertical = 5th axis
	
	Triggers = 3rd Axis	
	**/


    public static float sensitivity = 0.10f;
    public int controllerNumber = 1;
    private float rumbleStrength = 0.2f;
	
    public string ctrlString = "1";
    PlayerIndex playerIndex = PlayerIndex.One;
    PlayerScript playerScript = null;

    private GamePadState newState;

	public GameObject overLapAnim;
	public GameObject overLapAnim2;
	public GameObject overLapAnim3;
	public GameObject overLapAnim4;
	
	Texture2D background;
	Transform bgTrans;
	Transform thisTrans;
	Vector3 screenPos;
	Vector2 pixelUV;
	Camera renderCam;

    private float lastAnalogX = 0;
    private float lastAnalogY = 0;
	
	public Color color1 = new Color(0.9529F, 0.6392F, 0.0627F, 1.0F);
	public Color color2 = new Color(0.88627F, 0.2902F, 0.10588F, 1.0F);
	public Color color3 = new Color(0.08235F, 0.5412F, 0.3333F, 1.0F);
	public Color color4 = new Color(0.5647F, 0.1137F, 0.22745F, 1.0F);

    public const float LEVEL_EDGE_LEFT = -4.7f;
    public const float LEVEL_EDGE_RIGHT = 4.7f;
    public const float LEVEL_EDGE_TOP = 4.5f;
    public const float LEVEL_EDGE_BOTTOM = -4.5f;

	// Use this for initialization
	void Start () {
        
        ctrlString = System.Convert.ToString(controllerNumber);
        playerScript = (PlayerScript)this.gameObject.GetComponent("PlayerScript");
        playerScript.playerNumber = controllerNumber;
        switch (controllerNumber)
        {
            case 1: playerIndex = PlayerIndex.One; break;
            case 2: playerIndex = PlayerIndex.Two; break;
            case 3: playerIndex = PlayerIndex.Three; break;
            case 4: playerIndex = PlayerIndex.Four; break;
        }
        newState = GamePad.GetState(playerIndex);

		thisTrans = gameObject.transform;
		renderCam = GameObject.Find("bg_renderCam").camera;
	}
	
	// Update is called once per frame
	void Update () {

        float targetVibrationL = 0;
        float targetVibrationR = 0;
        
        newState = GamePad.GetState(playerIndex);

        if (!playerScript.dead)
        {
            //Check for color under player

            RaycastHit hit;
            if (Physics.Raycast(thisTrans.position, Vector3.forward, out hit, 100.0F))
            {
                if (hit.collider.gameObject.layer == 8)
                {
                    pixelUV = hit.textureCoord;
                    background = GetRTPixels(renderCam.targetTexture, pixelUV.x, pixelUV.y);
                    playerScript.backgroundColor = background.GetPixel(1, 1);
                    Color backgroundColor = playerScript.backgroundColor;
                    Color lastBackgroundColor = playerScript.lastBackgroundColor;
                    if (lastBackgroundColor != backgroundColor)
                    {
                        playerScript.lastBackgroundColor = backgroundColor;
                        lastBackgroundColor = backgroundColor;

                        GameObject toSpawn = null;
                        if (cCloseTo(backgroundColor, color1)) { toSpawn = overLapAnim; }
                        if (cCloseTo(backgroundColor, color2)) { toSpawn = overLapAnim2; }
                        if (cCloseTo(backgroundColor, color3)) { toSpawn = overLapAnim3; }
                        if (cCloseTo(backgroundColor, color4)) { toSpawn = overLapAnim4; }

                        if (toSpawn != null)
                        {
                            GameObject overlap = Instantiate(toSpawn, new Vector3(thisTrans.position.x, thisTrans.position.y + 30, thisTrans.position.z), Quaternion.identity) as GameObject;
                            overlap.transform.localScale = new Vector3(.2f, .2f, .2f);
                        }

                    }
                }
                else { Debug.Log("Hit something else!"); }
            }
            else { Debug.Log("Didn't hit anything!"); }

            //------------

            if (newState.Buttons.A == ButtonState.Pressed)
            { // A
                int xloc = (int)Mathf.Ceil(this.transform.position.x / AreaScript.tileSizeX);
                int yloc = (int)Mathf.Ceil(this.transform.position.y / AreaScript.tileSizeY);
                string color = GameController.GetColorName(playerScript.backgroundColor);
                if (color != "white" && color != playerScript.currentColor)
                {
                    if (playerScript.CheckChangeColor())
                    {
                        GameController.ChangeColor(color, playerScript);
                    }
                }
            }
            if (newState.Buttons.B == ButtonState.Pressed)
            { // B
                playerScript.Attack(lastAnalogX, lastAnalogY);
            }
            if (newState.Buttons.X == ButtonState.Pressed)
            { // X
                playerScript.Attack(lastAnalogX, lastAnalogY);
            }

            //analog sticks
            float translation1 = newState.ThumbSticks.Left.Y * Time.deltaTime * 20 * sensitivity;
            transform.Translate(0, translation1, 0);
            float translation2 = newState.ThumbSticks.Left.X * Time.deltaTime * 20 * sensitivity;
            transform.Translate(translation2, 0, 0);

            if ((translation1 < -0.05 || translation1 > 0.05) || (translation2 < -0.05 || translation2 > 0.05))
            {
                lastAnalogX = translation1;
                lastAnalogY = translation2;
            }

            //Make sure player does not go out of bounds
            float x = transform.position.x;
            float y = transform.position.y;
            float z = transform.position.z;
            if (transform.position.x < LEVEL_EDGE_LEFT) x = LEVEL_EDGE_LEFT;
            if (transform.position.x > LEVEL_EDGE_RIGHT) x = LEVEL_EDGE_RIGHT;
            if (transform.position.y < LEVEL_EDGE_BOTTOM) y = LEVEL_EDGE_BOTTOM;
            if (transform.position.y > LEVEL_EDGE_TOP) y = LEVEL_EDGE_TOP;

            transform.position = new Vector3(x, y, z);

            //Reset set of colliding objects; will be rewritten to it afterwards if still there
            playerScript.collidingObjects.Clear();

            //Vibrate depending on distance to closest enemy
            float closeness = 0;
            float maxCloseness = 4;
            for (int i = 0; i < GameController.players.Count; i++)
            {
                PlayerScript player = GameController.players[i];
                if (player.playerNumber != controllerNumber)
                {
                    float newCloseness = maxCloseness - Vector2.Distance(this.gameObject.transform.position, player.gameObject.transform.position);
                    if (player.dead) newCloseness = 0;
                    if (newCloseness > closeness) closeness = newCloseness;
                }
            }
            closeness = closeness / maxCloseness;

            targetVibrationL = closeness * rumbleStrength;
            targetVibrationR = closeness * rumbleStrength;

            
        }

        //DEATH RUMBLE!
        if (playerScript.deathRumbleTimer > 0)
        {
            targetVibrationL = 1; targetVibrationR = 1;
        }

        if (!playerScript.dead || playerScript.deathRumbleTimer > 0 || playerScript.livesLeft == 0)
        {
            GamePad.SetVibration(playerIndex, targetVibrationL, targetVibrationR);
        }

        //Restart
        if (Input.GetKey(KeyCode.Backspace)) Application.LoadLevel(Application.loadedLevel);
	}

	bool cCloseTo(Color fromColorm, Color toColor){
		/*print (fromColorm.GetHashCode()+" | "+toColor.GetHashCode());
		if(fromColorm.GetHashCode() - toColor.GetHashCode() > -50000000 && fromColorm.GetHashCode() - toColor.GetHashCode() < 50000000){
			return true;
		} else {
			return false;
		}*/
        if (GameController.GetColorName(fromColorm) == GameController.GetColorName(toColor)) return true;
        else return false;
	}
	
	Texture2D GetRTPixels(RenderTexture rt, float normalizedX, float normalizedY) {
		RenderTexture currentActiveRT = RenderTexture.active;
		RenderTexture.active = rt;
		Texture2D tex = new Texture2D(1, 1);
		tex.ReadPixels(new Rect(normalizedX*rt.width, (1 - normalizedY)*rt.height, 1, 1), 0, 0);
		tex.Apply();
		//Debug Texture to check color
		//GameObject.Find("debugPlane").renderer.material.mainTexture = tex;
		// 
		RenderTexture.active = currentActiveRT;
		return tex;
	}
}
