﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PlayerScript : MonoBehaviour {

    public Texture2D[] textures;
  //  public GUIText cooldownText;

    public float activationColorCooldown = 5f; //In seconds
    public float currentColorCooldown = 0f;
    public float tempColorChangeCooldown = 2.5f;
    public float colorRevertTimer = 0f;

    public string currentColor = "original";
    public string backupColor = "original";
    public bool dead = false;
    public int livesLeft = 1;
    public int playerNumber = 1;

    public float respawn_time = 2f;
    public float respawnTimer = 0f;

    public float death_rumble_time = 2f;
    public float deathRumbleTimer = 0f;

    public Color lastBackgroundColor;
    public Color backgroundColor;

    public GameObject spawnArea;

    public float heartSpritesStartX = 0;
    public float heartSpritesStartY = 0;
    const float heartspacing = 0.35f;
    public float colorBarStartX = 0;
    public float colorBarStartY = 0;

    public GameObject heart;
    public GameObject deadHeart;

    private GameObject colorChangeBar = null;

    public GameObject setColorChangeBarIncomplete;
    public GameObject setColorChangeBarComplete;

    public List<Collider2D> collidingObjects = new List<Collider2D>();
    private List<GameObject> heartSprites = new List<GameObject>();

    public GameObject deathParticle;
    public GameObject colorChangeParticle;
    public GameObject attackParticle;

	// Use this for initialization
	void Start () {
        for (int i = 0; i < livesLeft + 1; i++)
        {
            GameObject newHeart = (GameObject) GameObject.Instantiate(heart, new Vector3(heartSpritesStartX + i * heartspacing, heartSpritesStartY, 0), Quaternion.identity);
            heartSprites.Add(newHeart);
        }

        CreateColorBar(true);
	}

    public void CreateColorBar(bool complete = false)
    {
        if (colorChangeBar != null) Destroy(colorChangeBar);

        GameObject original = setColorChangeBarComplete;
        if (!complete) original = setColorChangeBarIncomplete;
        colorChangeBar = (GameObject)GameObject.Instantiate(original, new Vector3(colorBarStartX, colorBarStartY, 0), Quaternion.identity);
        if (!complete) colorChangeBar.transform.localScale = new Vector3(0.001f, 1, 1);
    }

    // Update is called once per frame
    void Update()
    {
        if (!dead)
        {
            //Color change cooldown
            if (currentColorCooldown > 0)
            {
                currentColorCooldown -= Time.deltaTime;
                colorChangeBar.transform.localScale = new Vector3(1 - (currentColorCooldown / activationColorCooldown), 1, 1);

                if (currentColorCooldown <= 0)
                {
                    currentColorCooldown = 0;
                    CreateColorBar(true);
                }
            }
            

            //Revert temporary color changes
            if (colorRevertTimer > 0)
            {
                colorRevertTimer -= Time.deltaTime;

                if (colorRevertTimer <= 0)
                {
                    ChangeColor(backupColor, false);
                }
            }

            //Check if you're invisible
            SpriteRenderer sr = (SpriteRenderer)this.gameObject.GetComponent("SpriteRenderer");
            if (GameController.GetColorName(backgroundColor) == currentColor && currentColor != "white" && colorRevertTimer <= 0)
            {
                sr.enabled = false;
            }
            else sr.enabled = true;
        }
        else
        {
            if (respawnTimer > 0)
            {
                respawnTimer -= Time.deltaTime;
                if (respawnTimer <= 0)
                {
                    //Erase hearts
                    for (int i = 0; i < heartSprites.Count; i++)
                    {
                        Destroy(heartSprites[i]);
                    }
                    heartSprites.Clear();

                    //--------------------RESPAWN!------------------------
                    if (livesLeft >= 1)
                    {
                        GameObject newPlayer = (GameObject)GameObject.Instantiate(this.gameObject);
                        newPlayer.transform.localPosition = spawnArea.transform.position;
                        PlayerScript ps = (PlayerScript)newPlayer.GetComponent("PlayerScript");
                        ps.livesLeft = this.livesLeft - 1;
                        ps.dead = false;
                        ps.ChangeColor("original");
                        GameController.PlaySound("respawn");
                        GameController.ReplacePlayer(ps);
                    }
                }
            }
        }

        if (deathRumbleTimer > 0f) deathRumbleTimer -= Time.deltaTime;
	}

    public void ChangeColor(string targetColor, bool real = true)
    {
        if (!dead)
        {
            if (real)
            {
                colorRevertTimer = 0;
                this.currentColor = targetColor;
            }
            else
            {
                backupColor = currentColor;
            }

            SpriteRenderer sr = (SpriteRenderer)this.gameObject.GetComponent("SpriteRenderer");

            for (int i = 0; i < textures.Length; i++)
            {
                if (textures[i].name == "character_" + targetColor)
                {
                    sr.sprite = Sprite.Create(textures[i], new Rect(0, 0, textures[i].width, textures[i].height),
                        new Vector2(0.5f, 0.5f));
                }
            }
        }
    }

    public void CreateChangeAnimation()
    {
        float x = this.gameObject.transform.position.x;
        float y = this.gameObject.transform.position.y;
        GameObject.Instantiate(colorChangeParticle, new Vector3(x, y, -7), Quaternion.identity);
    }

    public void CreateAttackAnimation(float lastX, float lastY)
    {
        float x = this.gameObject.transform.position.x;
        float y = this.gameObject.transform.position.y;
        GameObject goj = (GameObject) GameObject.Instantiate(attackParticle, new Vector3(x, y, -7), Quaternion.LookRotation(new Vector3(lastX, lastY), new Vector3(0, 0, -1)));
        goj.transform.parent = this.gameObject.transform;
    }

    //Check for permission to use change color button
    public bool CheckChangeColor()
    {
        if (this.currentColorCooldown > 0 || dead) return false;
        else
        {
            this.currentColorCooldown = activationColorCooldown;
            CreateColorBar(false);
            return true;
        }
    }

    public void Attack(float lastAnalogX, float lastAnalogY)
    {
        if (colorRevertTimer > 0) return; //Don't attack while attacking
        colorRevertTimer = tempColorChangeCooldown;
        ChangeColor("white", false);
        GameController.PlaySound("hit");
        CreateAttackAnimation(lastAnalogX, lastAnalogY);

        for (int i = 0; i < collidingObjects.Count; i++)
        {
            PlayerScript otherPlayer = (PlayerScript)collidingObjects[i].gameObject.GetComponent("PlayerScript");
            if (otherPlayer != null)
            {
                otherPlayer.Die();
            }
        }
    }

    public void Die()
    {
        if (!dead)
        {
            ChangeColor("black");
            dead = true;
            GameController.PlaySound("death");

            heartSprites.Add((GameObject) GameObject.Instantiate(deadHeart, new Vector3(heartSpritesStartX + livesLeft * heartspacing, heartSpritesStartY, 0), Quaternion.identity));

            float x = this.gameObject.transform.position.x;
            float y = this.gameObject.transform.position.y;
            GameObject.Instantiate(deathParticle, new Vector3(x, y, -7), Quaternion.identity);

            //Become visible again if you weren't, and go to background texture
            SpriteRenderer sr = (SpriteRenderer)this.gameObject.GetComponent("SpriteRenderer");
            sr.enabled = true;
            this.transform.Translate(new Vector3(0, 30, 0));

            respawnTimer = respawn_time;
            deathRumbleTimer = death_rumble_time;

			GameController.PlayerDied();
        }
    }

    public void OnTriggerEnter2D(Collider2D col)
    {
        if (!collidingObjects.Contains(col))
        {
            collidingObjects.Add(col);
        }
    }

    public void OnTriggerStay2D(Collider2D col)
    {
        if (!collidingObjects.Contains(col))
        {
            collidingObjects.Add(col);
        }
    }
}
